
import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;

public class Osmosis extends JPanel {
        
    int membraneLeft = 245;
    int membraneWidth = 20;
    int membraneRight = membraneLeft + membraneWidth;
    int membraneHeight = 100;
    int membraneGap = 40;
    int redRadius = 5, blueRadius = 9;
    double randomMoveDist = 5;          // How much to move.

    LinkedList<Point> reds, blues;      // Centers of particles.
    int n = 20;                          // # reds
    int m = 10;                          // # blues
    int t = 100;                        // #time steps to simulate.

    int time = 0;                       // Current time.

    // GUI variables.
    JTextField nField = new JTextField (5);
    JTextField mField = new JTextField (5);
    JTextField tField = new JTextField (7);
    int sleepTime = 100;
    String currentMsg = "";
    String terminateMsg = "";
    Thread currentThread;



    // When the "go" button is pressed, make a thread and fire off a simulation run.

    void go ()
    {
        if (currentThread != null) {
            currentThread.interrupt ();
            currentThread = null;
        }
        
	currentThread = new Thread () {
		public void run () 
		{
		    simulate ();
		}
	    };
	currentThread.start ();
    }
    


    // Perform one simulation run.

    void simulate ()
    {
        reset ();
        while (time <= t) {   // Stop if it collides with the winch.
	    try {
		Thread.sleep (sleepTime);
	    }
	    catch (InterruptedException e) {
                break;
	    }

            nextStep ();
            repaint ();
            currentMsg = "time: " + time;
        }
	repaint ();
    }
    
    // Initialize variables at t=0

    void reset ()
    {
        // Read n and d.
        try {
            n = Integer.parseInt (nField.getText());
            m = Integer.parseInt (mField.getText());
            t = Integer.parseInt (tField.getText());
        }
        catch (NumberFormatException e) {
            n = 20;
            m = 10;
            t = 100;
        }

	// Add a bunch of reds on left.
	reds = new LinkedList<Point> ();
        for (int i=0; i<n; i++) {
	    int x = UniformRandom.uniform (10, membraneLeft-10);
	    int y = UniformRandom.uniform (10, 490);
            Point p = new Point (x,y);
            reds.add (p);
        }

	// Add blues on both sides.
	blues = new LinkedList<Point> ();
        for (int i=0; i<m; i++) {
	    double u = UniformRandom.uniform ();
	    int x=0;
	    if (u < 0.5) {
		x = UniformRandom.uniform (10, membraneLeft-10);
	    }
	    else {
		x = UniformRandom.uniform (membraneRight+10, 490);
	    }
	    int y = UniformRandom.uniform (10, 490);
            Point p = new Point (x,y);
            blues.add (p);
        }
        
	// Reset.
        time = 0;
    }
    

    // Computation at each step.

    void nextStep ()
    {
        // Update current time.
        time += 1;
        
	// Move each blue point a random amount, but they can't cross.
	for (Point p: blues) {
	    double angle = UniformRandom.uniform (0.0, 2*Math.PI);
	    int x = (int) Math.round (randomMoveDist * Math.cos(angle));
	    int y = (int) Math.round (randomMoveDist * Math.sin(angle));
	    if (p.x <= membraneLeft) {
		if (p.x + x >= membraneLeft) {
		    continue;
		}
	    }
	    if (p.x >= membraneRight) {
		if (p.x + x <= membraneRight) {
		    continue;
		}
	    }
	    p.x = p.x + x;
	    p.y = p.y + y;
	}


	// Move each red point - they can cross underneath given height.
	for (Point p: reds) {
	    double angle = UniformRandom.uniform (0.0, 2*Math.PI);
	    int x = (int) Math.round (randomMoveDist * Math.cos(angle));
	    int y = (int) Math.round (randomMoveDist * Math.sin(angle));
	    // See if caught by membrane.
	    boolean badY = false;
	    int mh = 2*membraneGap;
	    int whichmh = mh;
	    while (mh <= 500+2*membraneGap) {
		if ((p.y <= mh) && (p.y >= mh-membraneGap)) {
		    badY = true;
		    whichmh = mh;
		    break;
		}
		mh += 2*membraneGap;
	    }
	    if (p.x <= membraneLeft) {
		if ( (p.x + x >= membraneLeft) && (badY) ) {
		    continue;
		}
	    }
	    if (p.x >= membraneRight) {
		if ( (p.x + x <= membraneRight) && (badY) ) {
		    continue;
		}
	    }
	    if ( (p.x >= membraneLeft) && (p.x <= membraneRight) && (badY) ) {
		if ( (p.y + y <= whichmh) && (p.y + y >= whichmh-membraneGap) ) {
		    continue;
		}
	    }
	    p.x = p.x + x;
	    p.y = p.y + y;
	}
    }



    ////////////////////////////////////////////////////////////////////////
    // GUI stuff


    public void paintComponent (Graphics g)
    {
	super.paintComponent (g);
	Dimension D = this.getSize ();

	// Background.
	g.setColor (Color.white);
	g.fillRect (0,0, D.width, D.height);
        
	// Draw membrane.
	g.setColor (Color.orange);
	int mh = 2 * membraneGap;
	while (mh <= D.height+2*membraneGap) {
	    g.fillRect (membraneLeft, D.height-mh, membraneWidth, membraneGap);
	    mh += 2 * membraneGap;
	}

        if ((reds == null) || (blues == null)) {
            return;
        }

        // Draw reds
        g.setColor (Color.red);
        for (Point p: reds) {
            // Draw circle centered at p.x,p.y
            g.fillOval (p.x-redRadius,D.height-p.y-redRadius, 2*redRadius,2*redRadius);
        }

        // Draw blues
        g.setColor (Color.blue);
        for (Point p: blues) {
            // Draw circle centered at p.x,p.y
            g.fillOval (p.x-blueRadius,D.height-p.y-blueRadius, 2*blueRadius,2*blueRadius);
        }
        
        // Message.
	g.setColor (Color.black);
	g.drawString (currentMsg, 500, 20);
	g.drawString (terminateMsg, 500, 40);
    }
    


    JPanel makeBottomPanel ()
    {
        JPanel panel = new JPanel ();
        
        panel.add (new JLabel ("n:"));
        nField.setText (""+n);
        panel.add (nField);
        panel.add (new JLabel ("   "));
        panel.add (new JLabel ("m:"));
        mField.setText (""+m);
        panel.add (mField);
        panel.add (new JLabel ("   "));
        panel.add (new JLabel ("t:"));
        tField.setText (""+t);
        panel.add (tField);

        panel.add (new JLabel ("        "));
	JButton goB = new JButton ("Go");
	goB.addActionListener (
	   new ActionListener () {
		   public void actionPerformed (ActionEvent a)
		   {
		       go ();
		   }
           }
        );
	panel.add (goB);

        panel.add (new JLabel ("  "));
	JButton quitB = new JButton ("Quit");
	quitB.addActionListener (
	   new ActionListener () {
		   public void actionPerformed (ActionEvent a)
		   {
		       System.exit(0);
		   }
           }
        );
	panel.add (quitB);
        
        return panel;
    }


    void makeFrame ()
    {
        JFrame frame = new JFrame ();
        frame.setSize (700, 700);
        frame.setTitle ("Random Walk");
        Container cPane = frame.getContentPane();
        cPane.add (makeBottomPanel(), BorderLayout.SOUTH);
        cPane.add (this, BorderLayout.CENTER);
        frame.setVisible (true);
    }


    public static void main (String[] argv)
    {
	Osmosis osmosis = new Osmosis ();
        osmosis.makeFrame ();
    }

}
