
import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;

public class ParticleGrid extends JPanel {
        
    int startX=300, startY=300;         // Starting point.
    int ballRadius = 5;                 // Radius of each particle.
    double randomMoveDist = 5;          // How much to move.
    double randomMoveProb = 0.5;        // 1-Pr[move uses forces]

    LinkedList<Point> points;           // Centers of particles.
    int n = 9;                          // #particles - from screen.
    int m = 3;                          // Side of grid
    int t = 100;                        // #time steps to simulate.

    HashSet<Integer>[][] grid;          // To assign particles to grid cells.

    int time = 0;                       // Current time.

    // GUI variables.
    JTextField nField = new JTextField (5);
    JTextField mField = new JTextField (5);
    JTextField tField = new JTextField (7);
    int sleepTime = 100;
    String currentMsg = "";
    String terminateMsg = "";
    Thread currentThread;
    DecimalFormat df =  new DecimalFormat ("##.####");

    // Stats. Here, the event is "all particles in leftmost column".
    double numEvents, numSamples;


    // When the "go" button is pressed, make a thread and fire off a simulation run.

    void go ()
    {
        if (currentThread != null) {
            currentThread.interrupt ();
            currentThread = null;
        }
        
	currentThread = new Thread () {
		public void run () 
		{
		    simulate ();
		}
	    };
	currentThread.start ();
    }
    


    // Perform one simulation run.

    void simulate ()
    {
        reset ();
        while (time <= t) {   
	    try {
		Thread.sleep (sleepTime);
	    }
	    catch (InterruptedException e) {
                break;
	    }

            nextStep ();
            currentMsg = "time: " + time;
        }

        terminateMsg = "P[E] = " + df.format (numEvents/numSamples);
	repaint ();
    }
    

    // Initialize variables at t=0

    void reset ()
    {
        // Read n and d.
        try {
            n = Integer.parseInt (nField.getText());
            m = Integer.parseInt (mField.getText());
            t = Integer.parseInt (tField.getText());
        }
        catch (NumberFormatException e) {
            n = 9;
            m = 3;
            t = 100;
        }


        // Make grid.
        grid = new HashSet [m][m];
        for (int i=0; i<m; i++) {
            for (int j=0; j<m; j++) {
                grid[i][j] = new HashSet<Integer>();
            }
        }

        // All points in top left.
        for (int k=0; k<n; k++) {
            grid[0][0].add (k);
        }
        
	// Reset.
        time = 0;
	repaint ();

        // Stats.
        numEvents = numSamples = 0;
    }
    

    // Computation at each step.

    void nextStep ()
    {
        // Update current time.
        time += 1;

        HashSet<Integer>[][] nextGrid = new HashSet [m][m];
        for (int i=0; i<m; i++) {
            for (int j=0; j<m; j++) {
                nextGrid[i][j] = new HashSet<Integer>();
            }
        }
        
        for (int i=0; i<m; i++) {
            for (int j=0; j<m; j++) {
                if (grid[i][j].size() > 0) {
                    // Move these randomly.
                    Point[] neighbors = getNeighbors (i,j);
                    for (int k: grid[i][j]) {
                        // Pick random neighbor.
                        int index = UniformRandom.uniform (0, neighbors.length-1);
                        Point p = neighbors[index];
                        nextGrid[p.x][p.y].add (k);
                    }
                }
            }
        }

        grid = nextGrid;
        this.repaint ();

        // Stats.
        double totalParticles = 0;
        // Now add along one column.
        for (int i=0; i<m; i++) {
            totalParticles += grid[i][0].size();
        }
        if (totalParticles == n) {
            // All in this column.
            numEvents ++;
        }
        
        numSamples ++;
    }



    Point[] getNeighbors (int i, int j)
    {
        LinkedList<Point> neighbors = new LinkedList<Point> ();

        // Neighbors above (i-1)
        if ((i > 0) && (j > 0)) {  
            neighbors.add (new Point (i-1,j-1));  // NW
        }
        if ((i > 0) && (j < m-1)) {
            neighbors.add (new Point (i-1,j+1));  // NE
        }
        if (i > 0) {
            neighbors.add (new Point (i-1,j));    // m
        }
        if ((i < m-1) && (j > 0)) {
            neighbors.add (new Point (i+1,j-1));  // SW
        }
        if ((i < m-1) && (j < m-1)) {
            neighbors.add (new Point (i+1,j+1));  // SE
        }
        if (i < m-1) {
            neighbors.add (new Point (i+1,j));    // S
        }
        if (j > 0) {
            neighbors.add (new Point (i,j-1));    // W
        }
        if (j < m-1) {
            neighbors.add (new Point (i,j+1));    // E
        }

        Point[] neighborArray = new Point [neighbors.size()];
        int index = 0;
        for (Point p: neighbors) {
            neighborArray[index] = p;
            index ++;
        }

        return neighborArray;
    }
    

    void fastForward ()
    {
        while (time <= t) {   
            nextStep ();
        }

        currentMsg = "time: " + time;
        terminateMsg = "P[E] = " + df.format (numEvents/numSamples);
	repaint ();
    }
    

    ////////////////////////////////////////////////////////////////////////
    // GUI stuff


    public void paintComponent (Graphics g)
    {
	super.paintComponent (g);
	Dimension D = this.getSize ();

	// Background.
	g.setColor (Color.white);
	g.fillRect (0,0, D.width, D.height);
        
        // Draw grid.
        g.setColor (Color.gray);
        int inset = 30;
        int span = D.height - 2*inset;
        int d = span / m;
        for (int i=0; i<=m; i++) {
            // Top left.
            int x = inset;
            int y = D.height - (inset + i*d);
            g.drawLine (x,y, x+span,y);
        }
        for (int j=0; j<=m; j++) {
            int x = inset + j*d;
            int y = inset;
            g.drawLine (x,D.height-y, x,D.height-span-inset);
        }
        

        g.setColor (Color.red);
        // Draw points.
        for (int i=0; i<m; i++) {
            for (int j=0; j<m; j++) {
                // Top left.
                if ((grid != null) && (grid[i][j].size() > 0)) {
                    int topLeftX = inset + j*d;
                    int topLeftY = inset + (m-i)*d;
                    for (int k=0; k<grid[i][j].size(); k++) {
                        // Draw this point.
                        double x1 = UniformRandom.uniform (0.0, (double) d);
                        double y1 = UniformRandom.uniform (0.0, (double) d);
                        int x = topLeftX + (int) x1;
                        int y = topLeftY - (int) y1;
                        g.fillOval (x-ballRadius,D.height-y-ballRadius, 2*ballRadius,2*ballRadius);
                    }
                }
            }
        }
        
        
        // Message.
	g.setColor (Color.black);
	g.drawString (currentMsg, 450, 50);

	g.drawString (terminateMsg, 450, 80);
    }
    


    JPanel makeBottomPanel ()
    {
        JPanel panel = new JPanel ();
        
        panel.setLayout (new GridLayout (2,1));
        
        JPanel topPart = new JPanel ();

        topPart.add (new JLabel ("n:"));
        nField.setText (""+n);
        topPart.add (nField);
        topPart.add (new JLabel ("   "));
        topPart.add (new JLabel ("m:"));
        mField.setText (""+m);
        topPart.add (mField);
        topPart.add (new JLabel ("   "));
        topPart.add (new JLabel ("t:"));
        tField.setText (""+t);
        topPart.add (tField);

        panel.add (topPart);
        
        JPanel bottomPart = new JPanel ();

	JButton resetB = new JButton ("Reset");
	resetB.addActionListener (
	   new ActionListener () {
		   public void actionPerformed (ActionEvent a)
		   {
		       reset ();
		   }
           }
        );
	bottomPart.add (resetB);

        bottomPart.add (new JLabel ("  "));
	JButton nextB = new JButton ("Next");
	nextB.addActionListener (
	   new ActionListener () {
		   public void actionPerformed (ActionEvent a)
		   {
		       nextStep ();
		   }
           }
        );
	bottomPart.add (nextB);

        bottomPart.add (new JLabel ("  "));
	JButton goB = new JButton ("Go");
	goB.addActionListener (
	   new ActionListener () {
		   public void actionPerformed (ActionEvent a)
		   {
		       go ();
		   }
           }
        );
	bottomPart.add (goB);


        bottomPart.add (new JLabel ("  "));
	JButton ffB = new JButton ("FF");
	ffB.addActionListener (
	   new ActionListener () {
		   public void actionPerformed (ActionEvent a)
		   {
		       fastForward ();
		   }
           }
        );
	bottomPart.add (ffB);

        bottomPart.add (new JLabel ("  "));
	JButton quitB = new JButton ("Quit");
	quitB.addActionListener (
	   new ActionListener () {
		   public void actionPerformed (ActionEvent a)
		   {
		       System.exit(0);
		   }
           }
        );
	bottomPart.add (quitB);

        panel.add (bottomPart);

        return panel;
    }


    void makeFrame ()
    {
        JFrame frame = new JFrame ();
        frame.setSize (600, 500);
        frame.setTitle ("Particles in Grid");
        Container cPane = frame.getContentPane();
        cPane.add (makeBottomPanel(), BorderLayout.SOUTH);
        cPane.add (this, BorderLayout.CENTER);
        frame.setVisible (true);
    }


    public static void main (String[] argv)
    {
	ParticleGrid pGrid = new ParticleGrid ();
        pGrid.makeFrame ();
    }

}
