
import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;

public class RandomWalk extends JPanel {
        
    int startX=300, startY=300;         // Starting point.
    int ballRadius = 5;                 // Radius of each particle.
    double randomMoveDist = 5;          // How much to move.
    double randomMoveProb = 0.5;        // 1-Pr[move uses forces]

    LinkedList<Point> points;           // Centers of particles.
    int n = 1;                          // #particles - from screen.
    int d = 10;                         // Annular distance.
    int t = 100;                        // #time steps to simulate.

    int time = 0;                       // Current time.
    int avgX;                           // Avg X value.
    int avgY;                           // Avg Y value.
    boolean useForces = false;
    boolean simulationCompleted = false;

    int fr = 20;                        // Not used.

    // GUI variables.
    String[] simTypes = {"Model-1", "Model-2"};
    JComboBox simType = new JComboBox (simTypes);
    JTextField nField = new JTextField (5);
    JTextField dField = new JTextField (5);
    JTextField tField = new JTextField (7);
    int sleepTime = 100;
    String currentMsg = "";
    String terminateMsg = "";
    Thread currentThread;



    // When the "go" button is pressed, make a thread and fire off a simulation run.

    void go ()
    {
        if (currentThread != null) {
            currentThread.interrupt ();
            currentThread = null;
        }
        
	currentThread = new Thread () {
		public void run () 
		{
		    simulate ();
		}
	    };
	currentThread.start ();
    }
    


    // Perform one simulation run.

    void simulate ()
    {
        reset ();
        while (time <= t) {   // Stop if it collides with the winch.
	    try {
		Thread.sleep (sleepTime);
	    }
	    catch (InterruptedException e) {
                break;
	    }

            nextStep ();
            repaint ();
            currentMsg = "time: " + time;
        }

	histogram ();
	simulationCompleted = true;
	repaint ();
    }
    
    // Initialize variables at t=0

    void reset ()
    {
        // Read n and d.
        try {
            n = Integer.parseInt (nField.getText());
            d = Integer.parseInt (dField.getText());
            t = Integer.parseInt (tField.getText());
        }
        catch (NumberFormatException e) {
            n = 1;
            d = 10;
            t = 100;
        }

	// Type of simulation.
	String whichType = (String) simType.getSelectedItem ();
	if (whichType.equals ("Model-2")) {
	    useForces = true;
	}
	else {
	    useForces = false;
	}

        // Initialize points.
        points = new LinkedList<Point> ();
        for (int i=0; i<n; i++) {
            // Add a bunch of points at the starting point.
            Point p = new Point (startX, startY);
            points.add (p);
        }
        
	// Reset.
        time = 0;
	simulationCompleted = false;
    }
    

    // Computation at each step.

    void nextStep ()
    {
        // Update current time.
        time += 1;
        
	if (! useForces) {
	    // Move each point a random amount.
	    for (Point p: points) {
		double angle = UniformRandom.uniform (0.0, 2*Math.PI);
		int x = (int) Math.round (randomMoveDist * Math.cos(angle));
		int y = (int) Math.round (randomMoveDist * Math.sin(angle));
		p.x = p.x + x;
		p.y = p.y + y;
	    }
	    return;
	}

	// Otherwise, use repelling forces, separately computing in x,y directions.
	double[] xForces = new double [n];
	double[] yForces = new double [n];
	for (int i=0; i<n; i++) {
	    Point p = points.get (i);
	    xForces[i] = 0;
	    yForces[i] = 0;
	    for (Point q: points) {
		if (p == q) {
		    continue;
		}
		double dist = distance (p.x,p.y, q.x,q.y);
		// Magnitude of force
		double f = forceMagnitude (dist);
		// x and y components.
		double xComp=0, yComp=0;
		if (dist == 0) {
		    // Add random component.
		    double angle = UniformRandom.uniform (0.0, 2*Math.PI);
		    xComp = (int) Math.round (randomMoveDist * Math.cos(angle));
		    yComp = (int) Math.round (randomMoveDist * Math.sin(angle));
		}
		else {
		    xComp = - f * (q.x-p.x) / dist;
		    yComp = - f * (q.y-p.y) / dist;
		}
		xForces[i] += xComp;  
		yForces[i] += yComp;
	    }
	}

	// Normalize so that move distance is about the same as random.
	for (int i=0; i<n; i++) {
	    double mag = Math.sqrt(xForces[i]*xForces[i] + yForces[i]*yForces[i]);
	    if (mag != 0) {
		xForces[i] = xForces[i] * (randomMoveDist / mag);
		yForces[i] = yForces[i] * (randomMoveDist / mag);
	    }
	}

	// Now move.
	for (int i=0; i<n; i++) {
	    Point p = points.get (i);
	    if (UniformRandom.uniform() < randomMoveProb) {
		double angle = UniformRandom.uniform (0.0, 2*Math.PI);
		p.x += (int) Math.round (randomMoveDist * Math.cos(angle));
		p.y += (int) Math.round (randomMoveDist * Math.sin(angle));
	    }
	    else {
		p.x = p.x + (int) xForces[i];
		p.y = p.y + (int) yForces[i];
	    }
	}
	
	/* 
	   // Alternative approach: use Monte-Carlo with energy.
	for (Point p: points) {
	    double angle = UniformRandom.uniform (0.0, 2*Math.PI);
	    int x = (int) Math.round (randomMoveDist * Math.cos(angle));
	    int y = (int) Math.round (randomMoveDist * Math.sin(angle));
	    double currentSum = 0;
	    double nextSum = 0;
	    for (Point q: points) {
		double r = distance(p.x,p.y, q.x,q.y);
		if ( (p != q) &&  (r < fr) ) {
		    currentSum += force (p.x,p.y, q.x,q.y);
		    nextSum += force (p.x+x, p.y+y, q.x,q.y);
	    
		}
	    }
	    if (nextSum <= currentSum) {
		p.x = p.x + x;
		p.y = p.y + y;
	    }
	    }
	*/

    }


    double distance (double x1, double y1, double x2, double y2)
    {
	double d = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2);
	return Math.sqrt (d);
    }


    double forceMagnitude (double dist)
    {
	if (dist == 0) {
	    return Double.MAX_VALUE;
	}
	return 1.0 / (dist*dist);
    }


    void histogram ()
    {
	if (points.size() <= 10) {
	    return;
	}

	int numIntervals = 2*startX / d;
	int[] binCounts = new int [numIntervals];
	double totalX = 0, totalY = 0;
	double sqrSum = 0;
	for (Point p: points) {
	    int bin = p.x / d;
	    if ( (bin >= 0) && (bin < binCounts.length) ) {
		binCounts[bin] ++;
	    }
	    totalX += p.x; 
            totalY += p.y;
	    sqrSum += (p.x-startX)*(p.x-startX);
	}
	avgX = (int) (totalX / points.size());
	avgY = (int) (totalY / points.size());
	double varianceEst = sqrSum / points.size();
	double stdDevEst = Math.sqrt (varianceEst);
	if (points.size() > 10) {
	    terminateMsg = "Std-dev: " + String.format ("%6.2f",stdDevEst);
	    System.out.println ("Std deviation estimate: " + stdDevEst);
	}
	else {
	    terminateMsg = "";
	}

	// Plot histogram.
	Function histF = new Function ("Histogram");
	for (int i=0; i<binCounts.length; i++) {
	    if (binCounts[i] > 0) {
		histF.add (i*d-startX, binCounts[i]);
	    }
	}
	histF.show ();
    }


    ////////////////////////////////////////////////////////////////////////
    // GUI stuff


    public void paintComponent (Graphics g)
    {
	super.paintComponent (g);
	Dimension D = this.getSize ();

	// Background.
	g.setColor (Color.white);
	g.fillRect (0,0, D.width, D.height);
        
        // Draw circles.
        g.setColor (Color.gray);
        int r = d;
        while (startX-r > 0) {
            g.drawOval (startX-r,D.height-startY-r, 2*r, 2*r);
            r = r + d;
        }
        
        if (points == null) {
            return;
        }
        

        // Draw points.
        g.setColor (Color.red);
        for (Point p: points) {
            // Draw circle centered at p.x,p.y
            g.fillOval (p.x-ballRadius,D.height-p.y-ballRadius, 2*ballRadius,2*ballRadius);
        }
        
        // Message.
	g.setColor (Color.black);
	g.drawString (currentMsg, 500, 20);

	if ((! simulationCompleted) || (points.size() <= 10)) {
	    return;
	}

	// Draw line across.
	g.drawLine (0,D.height-startY, D.width,D.height-startY);
	// Draw center.
	g.fillOval (avgX-ballRadius,D.height-startY-ballRadius,
		    2*ballRadius, 2*ballRadius);

	g.drawString (terminateMsg, 500, 40);
    }
    


    JPanel makeBottomPanel ()
    {
        JPanel panel = new JPanel ();
        
        panel.add (new JLabel ("n:"));
        nField.setText (""+n);
        panel.add (nField);
        panel.add (new JLabel ("   "));
        panel.add (new JLabel ("d:"));
        dField.setText (""+d);
        panel.add (dField);
        panel.add (new JLabel ("   "));
        panel.add (new JLabel ("t:"));
        tField.setText (""+t);
        panel.add (tField);

        panel.add (new JLabel ("        "));
	panel.add (simType);

	/*
        panel.add (new JLabel ("        "));
	JButton resetB = new JButton ("Reset");
	resetB.addActionListener (
	   new ActionListener () {
		   public void actionPerformed (ActionEvent a)
		   {
		       reset ();
		   }
           }
        );
	panel.add (resetB);
	*/

        panel.add (new JLabel ("  "));
	JButton goB = new JButton ("Go");
	goB.addActionListener (
	   new ActionListener () {
		   public void actionPerformed (ActionEvent a)
		   {
		       go ();
		   }
           }
        );
	panel.add (goB);

        panel.add (new JLabel ("  "));
	JButton quitB = new JButton ("Quit");
	quitB.addActionListener (
	   new ActionListener () {
		   public void actionPerformed (ActionEvent a)
		   {
		       System.exit(0);
		   }
           }
        );
	panel.add (quitB);
        
        return panel;
    }


    void makeFrame ()
    {
        JFrame frame = new JFrame ();
        frame.setSize (700, 700);
        frame.setTitle ("Random Walk");
        Container cPane = frame.getContentPane();
        cPane.add (makeBottomPanel(), BorderLayout.SOUTH);
        cPane.add (this, BorderLayout.CENTER);
        frame.setVisible (true);
    }


    public static void main (String[] argv)
    {
	RandomWalk randWalk = new RandomWalk ();
        randWalk.makeFrame ();
    }

}
